import numpy
from PyQt5 import QtWidgets

from .i_graph_mod_1d import cwidg_central as cwidg_pwd
from .interactive_matrix import cwidg_central as cwidg_matrix
from .FUNCTIONS import get_layout_rciftab_obj, del_layout
from .w_item_constr import w_for_item_constr

def w_for_pd2d_proc(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):
    w_for_item_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread)
    del_layout(layout_3)

    stack_widg = QtWidgets.QStackedWidget()
    
    lay_grid = QtWidgets.QGridLayout()
    lay_grid.addWidget(QtWidgets.QLabel("sum"), 0, 1)
    lay_grid.addWidget(QtWidgets.QLabel("diff."), 0, 2)
    lay_grid.addWidget(QtWidgets.QLabel("up"), 0, 3)
    lay_grid.addWidget(QtWidgets.QLabel("down"), 0, 4)
    lay_grid.addWidget(QtWidgets.QLabel("projection"), 1, 0)
    lay_grid.addWidget(QtWidgets.QLabel("maxtix model"), 2, 0)
    lay_grid.addWidget(QtWidgets.QLabel("maxtix exp"), 3, 0)
    lay_grid.addWidget(QtWidgets.QLabel("maxtix exp - model"), 4, 0)
    lay_grid.addWidget(QtWidgets.QLabel("maxtix model (gn)"), 5, 0)
    lay_grid.addWidget(QtWidgets.QLabel("maxtix exp (gn)"), 6, 0)
    lay_grid.addWidget(QtWidgets.QLabel("maxtix exp - model (gn)"), 7, 0)


    _rb_1 = QtWidgets.QRadioButton()
    _rb_1.toggled.connect(lambda: stack_widg.setCurrentIndex(0))
    lay_grid.addWidget(_rb_1, 1, 1)
    _rb_1.setChecked(True)
    _rb_2 = QtWidgets.QRadioButton()
    _rb_2.toggled.connect(lambda: stack_widg.setCurrentIndex(1))
    lay_grid.addWidget(_rb_2, 2, 1)

    _rb_3 = QtWidgets.QRadioButton()
    _rb_3.toggled.connect(lambda: stack_widg.setCurrentIndex(2))
    lay_grid.addWidget(_rb_3, 1, 2)
    _rb_4 = QtWidgets.QRadioButton()
    _rb_4.toggled.connect(lambda: stack_widg.setCurrentIndex(3))
    lay_grid.addWidget(_rb_4, 2, 2)

    _rb_5 = QtWidgets.QRadioButton()
    _rb_5.toggled.connect(lambda: stack_widg.setCurrentIndex(4))
    lay_grid.addWidget(_rb_5, 1, 3)
    _rb_6 = QtWidgets.QRadioButton()
    _rb_6.toggled.connect(lambda: stack_widg.setCurrentIndex(5))
    lay_grid.addWidget(_rb_6, 2, 3)

    _rb_7 = QtWidgets.QRadioButton()
    _rb_7.toggled.connect(lambda: stack_widg.setCurrentIndex(6))
    lay_grid.addWidget(_rb_7, 1, 4)
    _rb_8 = QtWidgets.QRadioButton()
    _rb_8.toggled.connect(lambda: stack_widg.setCurrentIndex(7))
    lay_grid.addWidget(_rb_8, 2, 4)


    _rb_9 = QtWidgets.QRadioButton()
    _rb_9.toggled.connect(lambda: stack_widg.setCurrentIndex(8))
    lay_grid.addWidget(_rb_9, 3, 1)
    _rb_10 = QtWidgets.QRadioButton()
    _rb_10.toggled.connect(lambda: stack_widg.setCurrentIndex(9))
    lay_grid.addWidget(_rb_10, 3, 2)
    _rb_11 = QtWidgets.QRadioButton()
    _rb_11.toggled.connect(lambda: stack_widg.setCurrentIndex(10))
    lay_grid.addWidget(_rb_11, 3, 3)
    _rb_12 = QtWidgets.QRadioButton()
    _rb_12.toggled.connect(lambda: stack_widg.setCurrentIndex(11))
    lay_grid.addWidget(_rb_12, 3, 4)


    _rb_13 = QtWidgets.QRadioButton()
    _rb_13.toggled.connect(lambda: stack_widg.setCurrentIndex(12))
    lay_grid.addWidget(_rb_13, 4, 1)
    _rb_14 = QtWidgets.QRadioButton()
    _rb_14.toggled.connect(lambda: stack_widg.setCurrentIndex(13))
    lay_grid.addWidget(_rb_14, 4, 2)
    _rb_15 = QtWidgets.QRadioButton()
    _rb_15.toggled.connect(lambda: stack_widg.setCurrentIndex(14))
    lay_grid.addWidget(_rb_15, 4, 3)
    _rb_16 = QtWidgets.QRadioButton()
    _rb_16.toggled.connect(lambda: stack_widg.setCurrentIndex(15))
    lay_grid.addWidget(_rb_16, 4, 4)



    _rb_17 = QtWidgets.QRadioButton()
    _rb_17.toggled.connect(lambda: stack_widg.setCurrentIndex(16))
    lay_grid.addWidget(_rb_17, 5, 1)

    _rb_18 = QtWidgets.QRadioButton()
    _rb_18.toggled.connect(lambda: stack_widg.setCurrentIndex(17))
    lay_grid.addWidget(_rb_18, 5, 2)

    _rb_19 = QtWidgets.QRadioButton()
    _rb_19.toggled.connect(lambda: stack_widg.setCurrentIndex(18))
    lay_grid.addWidget(_rb_19, 5, 3)

    _rb_20 = QtWidgets.QRadioButton()
    _rb_20.toggled.connect(lambda: stack_widg.setCurrentIndex(19))
    lay_grid.addWidget(_rb_20, 5, 4)


    _rb_21 = QtWidgets.QRadioButton()
    _rb_21.toggled.connect(lambda: stack_widg.setCurrentIndex(20))
    lay_grid.addWidget(_rb_21, 6, 1)

    _rb_22 = QtWidgets.QRadioButton()
    _rb_22.toggled.connect(lambda: stack_widg.setCurrentIndex(21))
    lay_grid.addWidget(_rb_22, 6, 2)

    _rb_23 = QtWidgets.QRadioButton()
    _rb_23.toggled.connect(lambda: stack_widg.setCurrentIndex(22))
    lay_grid.addWidget(_rb_23, 6, 3)

    _rb_24 = QtWidgets.QRadioButton()
    _rb_24.toggled.connect(lambda: stack_widg.setCurrentIndex(23))
    lay_grid.addWidget(_rb_24, 6, 4)


    _rb_25 = QtWidgets.QRadioButton()
    _rb_25.toggled.connect(lambda: stack_widg.setCurrentIndex(24))
    lay_grid.addWidget(_rb_25, 7, 1)

    _rb_26 = QtWidgets.QRadioButton()
    _rb_26.toggled.connect(lambda: stack_widg.setCurrentIndex(25))
    lay_grid.addWidget(_rb_26, 7, 2)

    _rb_27 = QtWidgets.QRadioButton()
    _rb_27.toggled.connect(lambda: stack_widg.setCurrentIndex(26))
    lay_grid.addWidget(_rb_27, 7, 3)

    _rb_28 = QtWidgets.QRadioButton()
    _rb_28.toggled.connect(lambda: stack_widg.setCurrentIndex(27))
    lay_grid.addWidget(_rb_28, 7, 4)

    _lay_1 = QtWidgets.QHBoxLayout()
    _lay_1.addLayout(lay_grid)
    _lay_1.addStretch(1)
    layout_3.addLayout(_lay_1)


    ttheta = obj.ttheta
    phi = obj.phi
    intensity_up = obj.intensity_up
    intensity_up_sigma = obj.intensity_up_sigma
    intensity_down = obj.intensity_down
    intensity_down_sigma = obj.intensity_down_sigma
    intensity_up_total = obj.intensity_up_total
    intensity_down_total = obj.intensity_down_total
    intensity_up_net = obj.intensity_up_net
    intensity_down_net = obj.intensity_down_net

    x = ttheta
    y = phi

    try:
        z_u = intensity_up_total.transpose()
        z_u_e = intensity_up.transpose()
        z_u_s_sq = (intensity_up_sigma.transpose())**2
        _z_1 = numpy.where(numpy.isnan(z_u_e), 0., z_u).sum(axis=0)
        _z_1_e = numpy.where(numpy.isnan(z_u_e), 0., z_u_e).sum(axis=0)
        _z_1_s = numpy.sqrt(numpy.where(numpy.isnan(z_u_e), 0., z_u_s_sq).sum(axis=0))
        _n_1 = numpy.where(numpy.isnan(z_u_e), 0., 1.).sum(axis=0)
        z_1_u = _z_1/_n_1
        z_1_u_e = _z_1_e/_n_1
        z_1_u_s = _z_1_s/_n_1
    except:
        return

    z_d = intensity_down_total.transpose()
    z_d_e = intensity_down.transpose()
    z_d_s_sq = (intensity_down_sigma.transpose())**2
    _z_1 = numpy.where(numpy.isnan(z_d_e), 0., z_d).sum(axis=0)
    _z_1_e = numpy.where(numpy.isnan(z_d_e), 0., z_d_e).sum(axis=0)
    _z_1_s = numpy.sqrt(numpy.where(numpy.isnan(z_d_e), 0., z_d_s_sq).sum(axis=0))
    _n_1 = numpy.where(numpy.isnan(z_d_e), 0., 1.).sum(axis=0)
    z_1_d = _z_1/_n_1
    z_1_d_e = _z_1_e/_n_1
    z_1_d_s = _z_1_s/_n_1

    z_sum = intensity_up_total.transpose()+intensity_down_total.transpose()
    z_sum_e = intensity_up.transpose()+intensity_down.transpose()
    z_sum_s_sq = (intensity_up_sigma.transpose())**2+(intensity_down_sigma.transpose())**2
    _z_1_e = numpy.where(numpy.isnan(z_sum_e), 0., z_sum_e).sum(axis=0)
    _z_1 = numpy.where(numpy.isnan(z_sum_e), 0., z_sum).sum(axis=0)
    _z_1_s = numpy.sqrt(numpy.where(numpy.isnan(z_sum_e), 0., z_sum_s_sq).sum(axis=0))
    _n_1 = numpy.where(numpy.isnan(z_sum_e), 0., 1.).sum(axis=0)
    z_1_sum = _z_1/_n_1
    z_1_sum_e = _z_1_e/_n_1
    z_1_sum_s = _z_1_s/_n_1

    z_diff = intensity_up_total.transpose()-intensity_down_total.transpose()
    z_diff_e = intensity_up.transpose()-intensity_down.transpose()
    _z_1_e = numpy.where(numpy.isnan(z_diff_e), 0., z_diff_e).sum(axis=0)
    _z_1 = numpy.where(numpy.isnan(z_diff_e), 0., z_diff).sum(axis=0)
    _n_1 = numpy.where(numpy.isnan(z_diff_e), 0., 1.).sum(axis=0)
    z_1_diff = _z_1/_n_1
    z_1_diff_e = _z_1_e/_n_1
    z_1_diff_s = _z_1_s/_n_1

    widg_matrix_u = cwidg_matrix()
    widg_matrix_u.plot_matrix(x, y, z_u)
    widg_matrix_u_e = cwidg_matrix()
    widg_matrix_u_e.plot_matrix(x, y, z_u_e)
    widg_matrix_u_e_m = cwidg_matrix()
    widg_matrix_u_e_m.plot_matrix(x, y, z_u_e-z_u)
    widg_proj_u = cwidg_pwd()
    np_xysm_1 = numpy.vstack((x, z_1_u_e, z_1_u_s, z_1_u)).transpose()
    widg_proj_u.plot_numpy_arrays(np_xysm_1)

    widg_matrix_d = cwidg_matrix()
    widg_matrix_d.plot_matrix(x, y, z_d)
    widg_matrix_d_e = cwidg_matrix()
    widg_matrix_d_e.plot_matrix(x, y, z_d_e)
    widg_matrix_d_e_m = cwidg_matrix()
    widg_matrix_d_e_m.plot_matrix(x, y, z_d_e-z_d)
    widg_proj_d = cwidg_pwd()
    np_xysm_1 = numpy.vstack((x, z_1_d_e, z_1_d_s, z_1_d)).transpose()
    widg_proj_d.plot_numpy_arrays(np_xysm_1)

    widg_matrix_sum = cwidg_matrix()
    widg_matrix_sum.plot_matrix(x, y, z_sum)
    widg_matrix_sum_e = cwidg_matrix()
    widg_matrix_sum_e.plot_matrix(x, y, z_sum_e)
    widg_matrix_sum_e_m = cwidg_matrix()
    widg_matrix_sum_e_m.plot_matrix(x, y, z_sum_e-z_sum)
    widg_proj_sum = cwidg_pwd()
    np_xysm_1 = numpy.vstack((x, z_1_sum_e, z_1_sum_s, z_1_sum)).transpose()
    widg_proj_sum.plot_numpy_arrays(np_xysm_1)

    widg_matrix_diff = cwidg_matrix()
    widg_matrix_diff.plot_matrix(x, y, z_diff)
    widg_matrix_diff_e = cwidg_matrix()
    widg_matrix_diff_e.plot_matrix(x, y, z_diff_e)
    widg_matrix_diff_e_m = cwidg_matrix()
    widg_matrix_diff_e_m.plot_matrix(x, y, z_diff_e-z_diff)
    widg_proj_diff = cwidg_pwd()
    np_xysm_1 = numpy.vstack((x, z_1_diff_e, z_1_diff_s, z_1_diff)).transpose()
    widg_proj_diff.plot_numpy_arrays(np_xysm_1)

    stack_widg.addWidget(widg_proj_sum)
    stack_widg.addWidget(widg_matrix_sum)
    stack_widg.addWidget(widg_proj_diff)
    stack_widg.addWidget(widg_matrix_diff)
    stack_widg.addWidget(widg_proj_u)
    stack_widg.addWidget(widg_matrix_u)
    stack_widg.addWidget(widg_proj_d)
    stack_widg.addWidget(widg_matrix_d)

    stack_widg.addWidget(widg_matrix_sum_e)
    stack_widg.addWidget(widg_matrix_diff_e)
    stack_widg.addWidget(widg_matrix_u_e)
    stack_widg.addWidget(widg_matrix_d_e)

    stack_widg.addWidget(widg_matrix_sum_e_m)
    stack_widg.addWidget(widg_matrix_diff_e_m)
    stack_widg.addWidget(widg_matrix_u_e_m)
    stack_widg.addWidget(widg_matrix_d_e_m)


    np_gamma, np_nu, l_int =obj.recalc_to_gamma_nu_grid()
    x_gn, y_gn = np_gamma, np_nu
    z_u_e_gn, z_d_e_gn, z_sum_e_gn, z_diff_e_gn, z_u_m_gn, z_d_m_gn, z_sum_m_gn, z_diff_m_gn = l_int

    widg_matrix_u_e_gn = cwidg_matrix()
    widg_matrix_u_e_gn.plot_matrix(x_gn, y_gn, z_u_e_gn)
    widg_matrix_d_e_gn = cwidg_matrix()
    widg_matrix_d_e_gn.plot_matrix(x_gn, y_gn, z_d_e_gn)
    widg_matrix_sum_e_gn = cwidg_matrix()
    widg_matrix_sum_e_gn.plot_matrix(x_gn, y_gn, z_sum_e_gn)
    widg_matrix_diff_e_gn = cwidg_matrix()
    widg_matrix_diff_e_gn.plot_matrix(x_gn, y_gn, z_diff_e_gn)

    widg_matrix_u_m_gn = cwidg_matrix()
    widg_matrix_u_m_gn.plot_matrix(x_gn, y_gn, z_u_m_gn)
    widg_matrix_d_m_gn = cwidg_matrix()
    widg_matrix_d_m_gn.plot_matrix(x_gn, y_gn, z_d_m_gn)
    widg_matrix_sum_m_gn = cwidg_matrix()
    widg_matrix_sum_m_gn.plot_matrix(x_gn, y_gn, z_sum_m_gn)
    widg_matrix_diff_m_gn = cwidg_matrix()
    widg_matrix_diff_m_gn.plot_matrix(x_gn, y_gn, z_diff_m_gn)

    widg_matrix_u_em_gn = cwidg_matrix()
    widg_matrix_u_em_gn.plot_matrix(x_gn, y_gn, z_u_e_gn - z_u_m_gn)
    widg_matrix_d_em_gn = cwidg_matrix()
    widg_matrix_d_em_gn.plot_matrix(x_gn, y_gn, z_d_e_gn - z_d_m_gn)
    widg_matrix_sum_em_gn = cwidg_matrix()
    widg_matrix_sum_em_gn.plot_matrix(x_gn, y_gn, z_sum_e_gn - z_sum_m_gn)
    widg_matrix_diff_em_gn = cwidg_matrix()
    widg_matrix_diff_em_gn.plot_matrix(x_gn, y_gn, z_diff_e_gn - z_diff_m_gn)

    stack_widg.addWidget(widg_matrix_sum_m_gn)
    stack_widg.addWidget(widg_matrix_diff_m_gn)
    stack_widg.addWidget(widg_matrix_u_m_gn)
    stack_widg.addWidget(widg_matrix_d_m_gn)

    stack_widg.addWidget(widg_matrix_sum_e_gn)
    stack_widg.addWidget(widg_matrix_diff_e_gn)
    stack_widg.addWidget(widg_matrix_u_e_gn)
    stack_widg.addWidget(widg_matrix_d_e_gn)

    stack_widg.addWidget(widg_matrix_sum_em_gn)
    stack_widg.addWidget(widg_matrix_diff_em_gn)
    stack_widg.addWidget(widg_matrix_u_em_gn)
    stack_widg.addWidget(widg_matrix_d_em_gn)

    stack_widg.setCurrentIndex(0)

    layout_3.addWidget(stack_widg)
    

    return




__author__ = 'ikibalin'
__version__ = "2019_09_10"

import os
import sys

import numpy

from PyQt5 import QtWidgets
from PyQt5 import QtGui
from PyQt5 import QtCore
 
import matplotlib
import matplotlib.backends.backend_qt5agg
import matplotlib.figure 
import matplotlib.pyplot
 

class cwind_central(QtWidgets.QMainWindow):
    def __init__(self, fname):
        super(cwind_central, self).__init__()
        self.title = "program 'Graph'"
        self.setWindowTitle(self.title)
        widg_central = cwidg_central(fname)
        self.setCentralWidget(widg_central)
        self.show()
        
class cwidg_central(QtWidgets.QWidget):
    def __init__(self, ffig_full = None):
        super(cwidg_central, self).__init__()
        self.init_layout_central(ffig_full)
        self.setLayout(self.layout_central)
        self.ffig_full = ffig_full
 
    def init_layout_central(self, ffig_full):
        
        lay_main = QtWidgets.QHBoxLayout()
        self.graph = Graph(self, width=5, height=4)
        

        lay_1 = QtWidgets.QVBoxLayout()
        _b_gm = QtWidgets.QPushButton("values to clipboard")
        _b_gm.clicked.connect(self.give_values)
        
        lay_1.addStretch(1)
        lay_1.addWidget(_b_gm)

        lay_main.addWidget(self.graph)
        lay_main.addLayout(lay_1)

        
        self.layout_central = lay_main
        
    def plot_file(self, x, y_exp, y_sig=None): 
        """
Temporary file. It will be deleted
        """
        self.graph.ax_pri.cla()       

        self.graph.data_x = x
        self.graph.data_y = y_exp
        self.graph.data_sy = y_sig
        
        self.graph.set_data_to_graph()

    def plot_lines(self, x, l_y=None, l_y_sig=None): 
        self.graph.ax_pri.cla()       

        self.graph.data_x = x
        if l_y is not None:
            self.graph.data_l_y = l_y
        if ((l_y_sig is not None) & (self.graph.data_l_y is not None)):
            if (len(l_y_sig) == len(self.graph.data_l_y)):
                self.graph.data_l_y_sig = l_y_sig
        
        self.graph.set_data_to_graph()


    def give_values(self):
        
        val_x = self.graph.data_x
        val_y = self.graph.data_y
        val_sy = self.graph.data_sy

        sval_x = [f"{_:.3f}" for _ in val_x] 
        sval_y = [" None" if (numpy.isnan(_) | (_ is None)) else f"{_:.3f}" for _ in val_y] 

        if val_sy is None:
            sval_sy = ["" for _ in val_y]
        else:
            sval_sy = [" None" if (numpy.isnan(_) | (_ is None)) else f"{_:.3f}" for _ in val_sy] 
        ls_out = [f"{_1:} {_2:} {_3:}"  for _1, _2, _3 in zip(sval_x, sval_y, sval_sy)]

        cb = QtWidgets.QApplication.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText("\n".join(ls_out), mode=cb.Clipboard)

class Graph(matplotlib.backends.backend_qt5agg.FigureCanvasQTAgg):
 
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = matplotlib.figure.Figure(figsize=(width, height), dpi = dpi)
        fig.subplots_adjust(left = 0.07,
                            right = 0.97,
                            top = 0.97,
                            bottom = 0.07,
                            wspace = 0.0,
                            hspace = 0.0)
        
        super(Graph, self).__init__(fig)
        
        self.info_press = (False, False)

        self.data_x = []
        self.data_y = []
        self.data_sy = []

        self.data_l_y = []
        self.data_l_y_sig = []

        self.control = parent
        self.figure = fig
        
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        self.ax_pri = fig.add_subplot(111)
        
        fig.canvas.mpl_connect("button_press_event", self.onclick)
        
    def set_data_to_graph(self):
        col_1 = "#000000"
        #self.ax_pri.plot(self.data_x, self.data_y, "-")
        _x = self.data_x
        _y = self.data_y
        _y_sig = self.data_sy

        l_y = self.data_l_y
        l_y_sig = self.data_l_y_sig
        if (len(l_y) != 0):
            if (len(l_y)==len(l_y_sig)):
                for _y, _y_sig in zip(l_y, l_y_sig):
                    if _y_sig is None:
                        self.ax_pri.plot(_x, _y, "k-", linewidth=1.0)
                    else:
                        self.ax_pri.errorbar(_x, _y, yerr = _y_sig, ecolor = col_1, fmt='o', color=col_1, linewidth = 0.5)
            else:
                for _y in l_y:
                    self.ax_pri.plot(_x, _y, "k-", linewidth=1.0)
        else:
            if _y_sig is None:
                self.ax_pri.plot(_x, _y, "k-", linewidth=1.0)
            else:
                self.ax_pri.plot(_x, _y_sig, "b-", linewidth=1.0)  
            #self.ax_pri.errorbar(self.data_x, self.data_y, yerr = self.data_sy, ecolor = col_1, fmt='o', color=col_1, linewidth = 0.5)
        self.draw()
        
    def onclick(self, event):
        if event.button == 3:
            #print(event.xdata, event.ydata)
            #self.data_x.append(event.xdata)
            #self.data_y.append(event.ydata)
            #self.set_data_to_graph()
            
            if self.info_press == (False, False):
                self.info_press = (True, False)
                self.xlim = [event.xdata]
                self.ylim = [event.ydata]
            elif self.info_press == (True, False):
                self.info_press = (True, True)
                self.xlim.append(event.xdata)
                self.ylim.append(event.ydata)
            if self.info_press == (True, True):
                self.info_press = (False, False)
                xlim = (min(self.xlim), max(self.xlim))
                ylim = (min(self.ylim), max(self.ylim))
                self.ax_pri.set_xlim(xlim)
                self.ax_pri.set_ylim(ylim)
                self.xlim = []
                self.ylim = []
                self.draw()
        elif event.button == 2:
                self.info_press == (False, False)
                x_min = min(self.data_x)
                x_max = max(self.data_x)
                if len(self.data_l_y) != 0:
                    l_y_min, l_y_max = [], []
                    for _y in self.data_l_y:
                        if isinstance(_y, numpy.ndarray):
                            print("here")
                            _y_min = min(_y[numpy.logical_not(numpy.isnan(_y))])
                            _y_max = max(_y[numpy.logical_not(numpy.isnan(_y))])
                        else:
                            _y_min = min([_1 for _1 in _y if _1 is not None])
                            _y_max = max([_1 for _1 in _y if _1 is not None])
                        l_y_min.append(_y_min)
                        l_y_max.append(_y_max)
                    y_min = min(l_y_min)
                    y_max = max(l_y_max)
                else:
                    y_min = min(self.data_y)
                    y_max = max(self.data_y)
                x_diff = x_max - x_min
                y_diff = y_max - y_min
                xy_min = min([x_min, y_min])
                xy_max = max([x_max, y_max])
                xy_diff = xy_max - xy_min
                xlim = (xy_min - 0.05*xy_diff, xy_max + 0.05*xy_diff)
                x_lim = (x_min - 0.05*x_diff, x_max + 0.05*x_diff)
                y_lim = (y_min - 0.05*y_diff, y_max + 0.05*y_diff)
                self.ax_pri.set_xlim(x_lim)
                self.ax_pri.set_ylim(y_lim)
                self.xlim = []
                self.ylim = []
                self.draw()
        else:
            self.info_press == (False, False)
            self.xlim = []
            self.ylim = []
            
        
        
 
if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    fname = "NaCaAlF_exp.out"
    fname = "oHoTi_p.dat"
    ex = cwind_central(fname)
    
    sys.exit(app.exec_())
    
    
    
"""
    def clean_fig(self):
        self.ldata = []
        self.ldata_simple = []
        self.pcanvas.figure.clf()
        self.pcanvas.make_plot()
        self.pcanvas.draw()
        
    def replot_fig(self):
        lhelp=['#005555','#BB5555','#550055','#55BB55','#555500','#5555BB']
        for iddata, ddata in enumerate(self.ldata_simple):
            lflags = [ (isinstance(hh1,(float, int))&isinstance(hh2,(float, int))&isinstance(hh3,(float, int))&isinstance(hh4,(float, int))) for hh1,hh2,hh3,hh4 in zip(ddata["IntUP"],ddata["sIntUP"],ddata["IntDOWN"],ddata["sIntDOWN"])]
            lx = [hh for hh, flag in zip(ddata["ttheta"], lflags) if flag]
            li_u_exp = [hh for hh, flag in zip(ddata["IntUP"], lflags) if flag]
            lsi_u_exp = [hh for hh, flag in zip(ddata["sIntUP"], lflags) if flag]
            li_d_exp = [hh for hh, flag in zip(ddata["IntDOWN"], lflags) if flag]
            lsi_d_exp = [hh for hh, flag in zip(ddata["sIntDOWN"], lflags) if flag]
            
            #li_u_exp = ddata["IntUP"]
            #lsi_u_exp = ddata["sIntUP"]
            #li_d_exp = ddata["IntDOWN"]
            #lsi_d_exp = ddata["sIntDOWN"]
            ldiff_exp = [hh1-hh2 for hh1, hh2 in zip(li_u_exp, li_d_exp)]
            lsdiff_exp = [(hh1**2+hh2**2)**0.5  for hh1, hh2 in zip(lsi_u_exp, lsi_d_exp)]
            lcolors=[lhelp[iddata] for hh in range(3)]
            self.pcanvas.plot(lx,[li_u_exp,lsi_u_exp,None],[li_d_exp,lsi_d_exp,None],[ldiff_exp,lsdiff_exp,None],None,lcolors)
        xmin = float(self.le_xmin.text())
        xmax = float(self.le_xmax.text())
        ymin = float(self.le_ymin.text())
        ymax = float(self.le_ymax.text())
        self.pcanvas.set_limits(xmin,xmax,ymin,ymax)          
        self.pcanvas.draw()
"""    


import numpy
from PyQt5 import QtWidgets

from .i_graph_mod_1d import cwidg_central as cwidg_pwd
from .FUNCTIONS import get_layout_rciftab_obj, del_layout
from .w_loop_constr import w_for_loop_constr

def w_for_pd_peak(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):
    w_for_loop_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread)
    del_layout(layout_3)

    np_x_1 = numpy.array(obj.ttheta, dtype = float)
    np_y_1 = numpy.array(obj.width_ttheta, dtype = float)

    np_xy_1 = numpy.vstack((np_x_1, np_y_1)).transpose()
    w_s_1 = cwidg_pwd()
    w_s_1.plot_numpy_arrays(np_xy_1, np_x_1)

    layout_3.addWidget(w_s_1)
    return 
    


__author__ = 'ikibalin'
__version__ = "2019_09_10"

import os
import sys
 
from PyQt5 import QtWidgets
from PyQt5 import QtGui
from PyQt5 import QtCore
 
import numpy
import matplotlib
import matplotlib.backends.backend_qt5agg
import matplotlib.figure 
import matplotlib.pyplot
 


class cwind_central(QtWidgets.QMainWindow):
    def __init__(self):
        super(cwind_central, self).__init__()
        self.title = "program 'Graph'"
        self.setWindowTitle(self.title)
        widg_central = cwidg_central()
        self.setCentralWidget(widg_central)
        self.show()



class cwidg_central(QtWidgets.QWidget):
    def __init__(self):
        super(cwidg_central, self).__init__()
        self.init_layout_central()
        self.setLayout(self.layout_central)
 
    def init_layout_central(self):
        
        lay_main = QtWidgets.QHBoxLayout()
        self.graph = Graph(self, width=5, height=4)
        

        lay_1 = QtWidgets.QVBoxLayout()
        _b_gm = QtWidgets.QPushButton("values to clipboard")
        _b_gm.clicked.connect(self.give_values)

        lay_1.addStretch(1)
        lay_1.addWidget(_b_gm)
        

        lay_main.addWidget(self.graph)
        lay_main.addLayout(lay_1)

        
        self.layout_central = lay_main
        
    def plot_file(self, x, l_y_mod, l_y_exp, l_y_sig):
        self.graph.ax_pri.cla()
        
        self.graph.data_x = x
        self.graph.data_l_y_mod = l_y_mod # len(l_y_mod) == len(l_y_exp) == len(l_y_sig)
        self.graph.data_l_y_exp = l_y_exp
        self.graph.data_l_y_sig = l_y_sig

        self.graph.set_data_to_graph()

    def give_values(self):
        val_x = self.graph.data_x
        l_y_mod = self.graph.data_l_y_mod
        l_y_exp = self.graph.data_l_y_exp
        l_y_sig = self.graph.data_l_y_sig
        

        sval_x = [f"{_:.3f}" for _ in val_x] 
        
        ls_out = []
        for y_mod, y_exp, y_sig in zip(l_y_mod, l_y_exp, l_y_sig):
            ls_out.append("     x    y_exp.     sigma   y_model")
            sval_x = [" None" if (numpy.isnan(_) | (_ is None)) else f"{_:.3f}" for _ in val_x] 
            sy_mod = [" None" if (numpy.isnan(_) | (_ is None)) else f"{_:.3f}" for _ in y_mod] 
            sy_exp = [" None" if (numpy.isnan(_) | (_ is None)) else f"{_:.3f}" for _ in y_exp] 
            sy_sig = [" None" if (numpy.isnan(_) | (_ is None)) else f"{_:.3f}" for _ in y_sig] 
            ls_out.extend([f"{_1:} {_2:} {_3:} {_4:}"  for _1, _2, _3, _4 in zip(sval_x, sy_exp, sy_sig, sy_mod)])
            ls_out.append("\n")

        cb = QtWidgets.QApplication.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText("\n".join(ls_out), mode=cb.Clipboard)


class Graph(matplotlib.backends.backend_qt5agg.FigureCanvasQTAgg):
    
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = matplotlib.figure.Figure(figsize=(width, height), dpi = dpi)
        fig.subplots_adjust(left = 0.07,
                            right = 0.97,
                            top = 0.97,
                            bottom = 0.07,
                            wspace = 0.0,
                            hspace = 0.0)
        
        super(Graph, self).__init__(fig)
        
        self.info_press = (False, False)

        self.data_x = []
        self.data_l_y_mod = []
        self.data_l_y_exp = []
        self.data_l_y_sig = []
        
        self.control = parent
        self.figure = fig
        
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        self.ax_pri = fig.add_subplot(111)
        
        fig.canvas.mpl_connect("button_press_event", self.onclick)
        
    def set_data_to_graph(self):
        col_1 = "#FF0000"
        col_2 = "#0000FF"
        col_3 = "#006400"

        x = self.data_x
        if len(x) == 0:
            return
        l_y_mod = self.data_l_y_mod
        l_y_exp = self.data_l_y_exp
        l_y_sig = self.data_l_y_sig
        l_colors = [col_1, col_2]
        l_n = max([l_y_mod[0].max(), (l_y_exp[0]+3*l_y_sig[0]).max(), (l_y_exp[0]-l_y_mod[0]+3*l_y_sig[0]).max()])
        for y_mod, y_exp, y_sig, col in zip(l_y_mod, l_y_exp, l_y_sig, l_colors):
            y_max =  max([y_mod.max(), (y_exp+3*y_sig).max(), (y_exp-y_mod+3*y_sig).max()])
            y_min =  min([y_mod.min(), (y_exp-3*y_sig).min(), (y_exp-y_mod-3*y_sig).min()])
            y_diff = y_max - y_min
            self.ax_pri.errorbar(x, y_exp+l_n-y_max, yerr = y_sig, ecolor = col_1,  fmt='.', color=col, linewidth = 0.5)
            self.ax_pri.plot(x, y_mod+l_n-y_max, "k-", linewidth=1.0)
            self.ax_pri.plot(x, y_exp-y_mod+l_n-y_max, "k-",linewidth=0.5)
            l_n -=  y_diff



        #ly1_before = self.data_4_y_exp
        #y_diff_max = max(ly1_before)
        #y_diff_min = min(ly1_before) - y_diff_max + y_down_min
        x_min = min(x)
        x_max = max(x)
        x_diff = x_max - x_min
        
        y_max = max([l_y_mod[0].max(), (l_y_exp[0]+3*l_y_sig[0]).max(), (l_y_exp[0]-l_y_mod[0]+3*l_y_sig[0]).max()])
        self.xlim_orig = (x_min-0.05*x_diff, x_max+0.05*x_diff)
        self.ylim_orig = (l_n, y_max)
        """
        self.ax_pri.plot(self.data_1_x, self.data_1_y_mod, "k-", linewidth=1.0)    
         
        self.ax_pri.errorbar(self.data_1_x, self.data_1_y_exp, yerr = self.data_1_sy_exp, ecolor = col_1, fmt='.', color=col_1, linewidth = 0.5)
        """ 
        self.draw()
        
    def onclick(self, event):
        if event.button == 3:
            
            if self.info_press == (False, False):
                self.info_press = (True, False)
                self.xlim = [event.xdata]
                self.ylim = [event.ydata]
            elif self.info_press == (True, False):
                self.info_press = (True, True)
                self.xlim.append(event.xdata)
                self.ylim.append(event.ydata)
            if self.info_press == (True, True):
                self.info_press = (False, False)
                xlim = (min(self.xlim), max(self.xlim))
                ylim = (min(self.ylim), max(self.ylim))
                self.ax_pri.set_xlim(xlim)
                self.ax_pri.set_ylim(ylim)
                self.xlim = []
                self.ylim = []
                self.draw()
        elif event.button == 2:
                self.info_press == (False, False)
                self.ax_pri.set_xlim(self.xlim_orig)
                self.ax_pri.set_ylim(self.ylim_orig)
                self.xlim = []
                self.ylim = []
                self.draw()
        else:
            self.info_press == (False, False)
            self.xlim = []
            self.ylim = []
            
 
if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    fname = "NaCaAlF_exp.out"
    ex = cwind_central()
    
    sys.exit(app.exec_())



import numpy
from PyQt5 import QtWidgets

from .i_graph_mod_1d import cwidg_central as cwidg_pwd

from .interactive_matrix import cwidg_central as cwidg_matrix
from .FUNCTIONS import get_layout_rciftab_obj, del_layout
from .w_item_constr import w_for_item_constr

def w_for_pd2d_meas(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):
    w_for_item_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread)
    del_layout(layout_3)

    stack_widg = QtWidgets.QStackedWidget()
    
    lay_grid = QtWidgets.QGridLayout()
    lay_grid.addWidget(QtWidgets.QLabel("sum"), 0, 1)
    lay_grid.addWidget(QtWidgets.QLabel("diff."), 0, 2)
    lay_grid.addWidget(QtWidgets.QLabel("up"), 0, 3)
    lay_grid.addWidget(QtWidgets.QLabel("down"), 0, 4)
    lay_grid.addWidget(QtWidgets.QLabel("projection"), 1, 0)
    lay_grid.addWidget(QtWidgets.QLabel("maxtix exp"), 2, 0)
    lay_grid.addWidget(QtWidgets.QLabel("in gamma nu"), 3, 0)

    _rb_1 = QtWidgets.QRadioButton()
    _rb_1.toggled.connect(lambda: stack_widg.setCurrentIndex(0))
    lay_grid.addWidget(_rb_1, 1, 1)
    _rb_1.setChecked(True)
    _rb_2 = QtWidgets.QRadioButton()
    _rb_2.toggled.connect(lambda: stack_widg.setCurrentIndex(1))
    lay_grid.addWidget(_rb_2, 2, 1)

    _rb_3 = QtWidgets.QRadioButton()
    _rb_3.toggled.connect(lambda: stack_widg.setCurrentIndex(2))
    lay_grid.addWidget(_rb_3, 1, 2)
    _rb_4 = QtWidgets.QRadioButton()
    _rb_4.toggled.connect(lambda: stack_widg.setCurrentIndex(3))
    lay_grid.addWidget(_rb_4, 2, 2)

    _rb_5 = QtWidgets.QRadioButton()
    _rb_5.toggled.connect(lambda: stack_widg.setCurrentIndex(4))
    lay_grid.addWidget(_rb_5, 1, 3)
    _rb_6 = QtWidgets.QRadioButton()
    _rb_6.toggled.connect(lambda: stack_widg.setCurrentIndex(5))
    lay_grid.addWidget(_rb_6, 2, 3)

    _rb_7 = QtWidgets.QRadioButton()
    _rb_7.toggled.connect(lambda: stack_widg.setCurrentIndex(6))
    lay_grid.addWidget(_rb_7, 1, 4)
    _rb_8 = QtWidgets.QRadioButton()
    _rb_8.toggled.connect(lambda: stack_widg.setCurrentIndex(7))
    lay_grid.addWidget(_rb_8, 2, 4)

    _rb_9 = QtWidgets.QRadioButton()
    _rb_9.toggled.connect(lambda: stack_widg.setCurrentIndex(8))
    lay_grid.addWidget(_rb_9, 3, 1)

    _rb_10 = QtWidgets.QRadioButton()
    _rb_10.toggled.connect(lambda: stack_widg.setCurrentIndex(9))
    lay_grid.addWidget(_rb_10, 3, 2)

    _rb_11 = QtWidgets.QRadioButton()
    _rb_11.toggled.connect(lambda: stack_widg.setCurrentIndex(10))
    lay_grid.addWidget(_rb_11, 3, 3)

    _rb_12 = QtWidgets.QRadioButton()
    _rb_12.toggled.connect(lambda: stack_widg.setCurrentIndex(11))
    lay_grid.addWidget(_rb_12, 3, 4)




    _lay_1 = QtWidgets.QHBoxLayout()
    _lay_1.addLayout(lay_grid)
    _lay_1.addStretch(1)

    layout_3.addLayout(_lay_1)

    ttheta = obj.ttheta
    phi = obj.phi
    intensity_up = obj.intensity_up
    intensity_up_sigma = obj.intensity_up_sigma
    intensity_down = obj.intensity_down
    intensity_down_sigma = obj.intensity_down_sigma



    x = ttheta
    y = phi

    try:
        z_u_e = intensity_up.transpose()
        z_u_s_sq = (intensity_up_sigma.transpose())**2
        _z_1_e = numpy.where(numpy.isnan(z_u_e), 0., z_u_e).sum(axis=0)
        _z_1_s = numpy.sqrt(numpy.where(numpy.isnan(z_u_e), 0., z_u_s_sq).sum(axis=0))
        _n_1 = numpy.where(numpy.isnan(z_u_e), 0., 1.).sum(axis=0)
        z_1_u_e = _z_1_e/_n_1
        z_1_u_s = _z_1_s/_n_1
    except:
        return

    z_d_e = intensity_down.transpose()
    z_d_s_sq = (intensity_down_sigma.transpose())**2
    _z_1_e = numpy.where(numpy.isnan(z_d_e), 0., z_d_e).sum(axis=0)
    _z_1_s = numpy.sqrt(numpy.where(numpy.isnan(z_d_e), 0., z_d_s_sq).sum(axis=0))
    _n_1 = numpy.where(numpy.isnan(z_d_e), 0., 1.).sum(axis=0)
    z_1_d_e = _z_1_e/_n_1
    z_1_d_s = _z_1_s/_n_1

    z_sum_e = intensity_up.transpose()+intensity_down.transpose()
    z_sum_s_sq = (intensity_up_sigma.transpose())**2+(intensity_down_sigma.transpose())**2
    _z_1_e = numpy.where(numpy.isnan(z_sum_e), 0., z_sum_e).sum(axis=0)
    _z_1_s = numpy.sqrt(numpy.where(numpy.isnan(z_sum_e), 0., z_sum_s_sq).sum(axis=0))
    _n_1 = numpy.where(numpy.isnan(z_sum_e), 0., 1.).sum(axis=0)
    z_1_sum_e = _z_1_e/_n_1
    z_1_sum_s = _z_1_s/_n_1

    z_diff_e = intensity_up.transpose()-intensity_down.transpose()
    _z_1_e = numpy.where(numpy.isnan(z_diff_e), 0., z_diff_e).sum(axis=0)
    _n_1 = numpy.where(numpy.isnan(z_diff_e), 0., 1.).sum(axis=0)
    z_1_diff_e = _z_1_e/_n_1
    z_1_diff_s = _z_1_s/_n_1

    widg_matrix_u_e = cwidg_matrix()
    widg_matrix_u_e.plot_matrix(x, y, z_u_e)
    widg_proj_u = cwidg_pwd()
    np_xys_1 = numpy.vstack((x, z_1_u_e, z_1_u_s)).transpose()
    widg_proj_u.plot_numpy_arrays(np_xys_1)

    widg_matrix_d_e = cwidg_matrix()
    widg_matrix_d_e.plot_matrix(x, y, z_d_e)
    widg_proj_d = cwidg_pwd()
    np_xys_1 = numpy.vstack((x, z_1_d_e, z_1_d_s)).transpose()
    widg_proj_d.plot_numpy_arrays(np_xys_1)


    widg_matrix_sum_e = cwidg_matrix()
    widg_matrix_sum_e.plot_matrix(x, y, z_sum_e)
    widg_proj_sum = cwidg_pwd()
    np_xys_1 = numpy.vstack((x, z_1_sum_e, z_1_sum_s)).transpose()
    widg_proj_sum.plot_numpy_arrays(np_xys_1)

    widg_matrix_diff_e = cwidg_matrix()
    widg_matrix_diff_e.plot_matrix(x, y, z_diff_e)
    widg_proj_diff = cwidg_pwd()
    np_xys_1 = numpy.vstack((x, z_1_diff_e, z_1_diff_s)).transpose()
    widg_proj_diff.plot_numpy_arrays(np_xys_1)

    stack_widg.addWidget(widg_proj_sum)
    stack_widg.addWidget(widg_matrix_sum_e)
    stack_widg.addWidget(widg_proj_diff)
    stack_widg.addWidget(widg_matrix_diff_e)
    stack_widg.addWidget(widg_proj_u)
    stack_widg.addWidget(widg_matrix_u_e)
    stack_widg.addWidget(widg_proj_d)
    stack_widg.addWidget(widg_matrix_d_e)


    np_gamma, np_nu, l_int = obj.recalc_to_gamma_nu_grid()
    x_gn, y_gn = np_gamma, np_nu
    z_u_e_gn, z_d_e_gn, z_sum_e_gn, z_diff_e_gn = l_int

    widg_matrix_u_e_gn = cwidg_matrix()
    widg_matrix_u_e_gn.plot_matrix(x_gn, y_gn, z_u_e_gn)
    widg_matrix_d_e_gn = cwidg_matrix()
    widg_matrix_d_e_gn.plot_matrix(x_gn, y_gn, z_d_e_gn)
    widg_matrix_sum_e_gn = cwidg_matrix()
    widg_matrix_sum_e_gn.plot_matrix(x_gn, y_gn, z_sum_e_gn)
    widg_matrix_diff_e_gn = cwidg_matrix()
    widg_matrix_diff_e_gn.plot_matrix(x_gn, y_gn, z_diff_e_gn)

    stack_widg.addWidget(widg_matrix_sum_e_gn)
    stack_widg.addWidget(widg_matrix_diff_e_gn)
    stack_widg.addWidget(widg_matrix_u_e_gn)
    stack_widg.addWidget(widg_matrix_d_e_gn)

    stack_widg.setCurrentIndex(0)

    layout_3.addWidget(stack_widg)

    return 




import numpy
from PyQt5 import QtWidgets

from .i_graph_mod_1d import cwidg_central as cwidg_pwd
from .FUNCTIONS import get_layout_rciftab_obj, del_layout
from .w_loop_constr import w_for_loop_constr

def w_for_pd_proc(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):
    w_for_loop_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread)
    del_layout(layout_3)

    np_x_1 = numpy.array(obj.ttheta, dtype = float)
    np_y_u_1 = numpy.array(obj.intensity_up, dtype = float)
    np_y_su_1 = numpy.array(obj.intensity_up_sigma, dtype = float)
    np_y_d_1 = numpy.array(obj.intensity_down, dtype = float)
    np_y_sd_1 = numpy.array(obj.intensity_down_sigma, dtype = float)
    np_y_b_1 = numpy.array(obj.intensity_bkg_calc, dtype = float)
    np_y_u_2 = numpy.array(obj.intensity_up_total, dtype = float)
    np_y_d_2 = numpy.array(obj.intensity_down_total, dtype = float)
    np_y_s_1 = numpy.array(obj.intensity, dtype = float)
    np_y_ss_1 = numpy.array(obj.intensity_sigma, dtype = float)
    np_y_m_1 = np_y_u_1 - np_y_d_1
    np_y_sm_1 = np_y_ss_1
    np_y_s_2 = numpy.array(obj.intensity_total, dtype = float)
    np_y_m_2 = numpy.array(obj.intensity_diff_total, dtype = float)

    np_xysm_1 = numpy.vstack((np_x_1, np_y_s_1, np_y_ss_1, np_y_s_2)).transpose()
    np_xb_1 = numpy.vstack((np_x_1, 2*np_y_b_1)).transpose()
    w_s_1 = cwidg_pwd()
    w_s_1.plot_numpy_arrays(np_xysm_1, np_xb_1)

    np_xysm_1 = numpy.vstack((np_x_1, np_y_m_1, np_y_sm_1, np_y_m_2)).transpose()
    w_s_2 = cwidg_pwd()
    w_s_2.plot_numpy_arrays(np_xysm_1)

    np_xysm_1 = numpy.vstack((np_x_1, np_y_u_1, np_y_su_1, np_y_u_2)).transpose()
    np_xb_1 = numpy.vstack((np_x_1, np_y_b_1)).transpose()
    w_s_3 = cwidg_pwd()
    w_s_3.plot_numpy_arrays(np_xysm_1, np_xb_1)

    np_xysm_1 = numpy.vstack((np_x_1, np_y_d_1, np_y_sd_1, np_y_d_2)).transpose()
    w_s_4 = cwidg_pwd()
    w_s_4.plot_numpy_arrays(np_xysm_1, np_xb_1)

    stack_widg = QtWidgets.QStackedWidget()
    stack_widg.addWidget(w_s_1)
    stack_widg.addWidget(w_s_2)
    stack_widg.addWidget(w_s_3)
    stack_widg.addWidget(w_s_4)
        
    lay_h = QtWidgets.QHBoxLayout()
    _rb_1 = QtWidgets.QRadioButton("sum")
    _rb_1.toggled.connect(lambda: stack_widg.setCurrentIndex(0))
    lay_h.addWidget(_rb_1)
    _rb_1.setChecked(True)
    _rb_2 = QtWidgets.QRadioButton("diff")
    _rb_2.toggled.connect(lambda: stack_widg.setCurrentIndex(1))
    lay_h.addWidget(_rb_2)
    _rb_3 = QtWidgets.QRadioButton("up")
    _rb_3.toggled.connect(lambda: stack_widg.setCurrentIndex(2))
    lay_h.addWidget(_rb_3)
    _rb_4 = QtWidgets.QRadioButton("down")
    _rb_4.toggled.connect(lambda: stack_widg.setCurrentIndex(3))
    lay_h.addWidget(_rb_4)
    lay_h.addStretch(1)

    layout_3.addLayout(lay_h)
    layout_3.addWidget(stack_widg)
    return 




import pycifstar
import os
import copy


L_ITEM_CLASS = []
L_LOOP_CLASS = []
L_DATA_CLASS = []
L_GLOBAL_CLASS = []

try:
    FLAG_CRYSPY = True

    from cryspy.common.cl_global_constr import GlobalConstr
    from cryspy.common.cl_data_constr import DataConstr

    from cryspy.cif_like.cl_crystal import Crystal
    from cryspy.cif_like.cl_pd import Pd
    from cryspy.cif_like.cl_pd2d import Pd2d
    from cryspy.cif_like.cl_diffrn import Diffrn

    from cryspy.pd2dcif_like.cl_pd2d_meas import Pd2dMeas
    from cryspy.pd2dcif_like.cl_pd2d_proc import Pd2dProc

    from cryspy.cif_like.cl_chi2 import Chi2 
    from cryspy.cif_like.cl_diffrn_radiation import DiffrnRadiation 
    from cryspy.cif_like.cl_extinction import Extinction 
    from cryspy.cif_like.cl_range import Range 
    from cryspy.cif_like.cl_setup import Setup 
    from cryspy.cif_like.cl_texture import Texture 
    from cryspy.cif_like.cl_diffrn_refln import DiffrnRefln, DiffrnReflnL 
    from cryspy.cif_like.cl_exclude import Exclude, ExcludeL 
    from cryspy.cif_like.cl_phase import Phase, PhaseL 

    from cryspy.corecif.cl_atom_site import AtomSiteL
    from cryspy.corecif.cl_atom_site_aniso import AtomSiteAnisoL
    from cryspy.corecif.cl_atom_type import AtomTypeL
    from cryspy.corecif.cl_cell import Cell
    from cryspy.corecif.cl_diffrn_orient_matrix import DiffrnOrientMatrix
    from cryspy.corecif.cl_refine_ls import RefineLs
    from cryspy.corecif.cl_refln import ReflnL

    from cryspy.magneticcif.cl_atom_site_moment import AtomSiteMomentL
    from cryspy.magneticcif.cl_atom_site_scat import AtomSiteScatL
    from cryspy.magneticcif.cl_atom_site_susceptibility import AtomSiteSusceptibilityL
    from cryspy.magneticcif.cl_atom_type_scat import AtomTypeScatL
    from cryspy.magneticcif.cl_refln_susceptibility import ReflnSusceptibilityL

    from cryspy.symcif.cl_space_group import SpaceGroup
    from cryspy.symcif.cl_space_group_symop import SpaceGroupSymop, SpaceGroupSymopL
    from cryspy.symcif.cl_space_group_wyckoff import SpaceGroupWyckoff, SpaceGroupWyckoffL

    from cryspy.pd1dcif_like.cl_pd_background import PdBackground, PdBackgroundL
    from cryspy.pd1dcif_like.cl_pd_instr_reflex_asymmetry import PdInstrReflexAsymmetry
    from cryspy.pd1dcif_like.cl_pd_instr_resolution import PdInstrResolution
    from cryspy.pd1dcif_like.cl_pd_peak import PdPeak, PdPeakL
    from cryspy.pd1dcif_like.cl_pd_meas import PdMeas, PdMeasL
    from cryspy.pd1dcif_like.cl_pd_proc import PdProc, PdProcL

    from cryspy.scripts.cl_rhochi import RhoChi
    L_ITEM_CLASS.extend([PdInstrResolution, PdInstrReflexAsymmetry,
                         PdProc, PdPeak, PdMeas, PdBackground,
                         SpaceGroup, SpaceGroupSymop, SpaceGroupWyckoff,
                         Pd2dMeas, Pd2dProc,
                         Chi2, DiffrnRadiation, 
                         Extinction, Range, Setup,
                         Texture, Cell, DiffrnOrientMatrix,
                         RefineLs, DiffrnRefln, Exclude, Phase
                         ])

    L_LOOP_CLASS.extend([PdProcL, PdPeakL, PdMeasL, PdBackgroundL, SpaceGroupSymopL, SpaceGroupWyckoffL, 
                         AtomSiteMomentL, AtomSiteScatL, AtomSiteSusceptibilityL,
                         AtomTypeScatL, ReflnSusceptibilityL, DiffrnReflnL, ExcludeL, 
                         PhaseL, AtomSiteL, AtomSiteAnisoL,
                         AtomTypeL,ReflnL])

    L_DATA_CLASS.extend([Crystal, Pd, Pd2d, Diffrn])

    L_GLOBAL_CLASS.append(RhoChi)

except ImportError:
    FLAG_CRYSPY = False

try:
    FLAG_MAGREF = True

    from magref.classes.cl_cryst_field import CrystField
    from magref.classes.cl_coefficient_stevens import CoefficientStevens
    from magref.classes.cl_coefficient_wybourne import CoefficientWybourne
    from magref.classes.cl_inelastic_parameter import InelasticParameter
    from magref.classes.cl_ion_type import IonType
    from magref.classes.cl_inelastic_background import InelasticBackgroundL
    from magref.classes.cl_meas_elastic import MeasElasticL
    from magref.classes.cl_meas_inelastic import MeasInelasticL
    from magref.classes.cl_meas_inelastic_peak import MeasInelasticPeakL
    from magref.classes.cl_point_site import PointSiteL
    from magref.classes.cl_point_type import PointTypeL
    from magref.classes.cl_refinement import RefinementL
    from magref.scripts.cl_cryst_ref import CrystRef

    L_ITEM_CLASS.extend([CoefficientStevens, CoefficientWybourne, InelasticParameter, IonType])
    L_LOOP_CLASS.extend([InelasticBackgroundL, MeasElasticL, MeasInelasticL, MeasInelasticPeakL, 
                         PointSiteL, PointTypeL, RefinementL])
    L_DATA_CLASS.extend([CrystField])
    L_GLOBAL_CLASS.append(CrystRef)

except ImportError:
    FLAG_MAGREF = False


try:
    FLAG_MEM = True

    from lib_mem.mem.cl_density_points_number import DensityPointsNumber
    from lib_mem.mem.cl_density_point import DensityPointL
    from lib_mem.mem.cl_density import Density
    from lib_mem.mem.cl_compting_parameters import ComputingParameters

    from lib_mem.scripts.cl_mem_tensor import MemTensor

    L_ITEM_CLASS.extend([ComputingParameters, DensityPointsNumber])
    L_LOOP_CLASS.extend([DensityPointL])
    L_DATA_CLASS.extend([Density])
    L_GLOBAL_CLASS.append(MemTensor)
except ImportError:
    FLAG_MEM = False


def is_in_rcif_block(rcif_block, cryspy_obj):
    ls_cryspy = cryspy_obj.to_cif.split("\n")
    l_name = [_.strip().split()[0] for _ in ls_cryspy if _.startswith("_")]
    l_flag = [rcif_block.is_value(_name) for _name in l_name]
    return any(l_flag), all(l_flag)
    

def rcif_to_cryspy(rcif):
    l_global_obj = []
    l_defined_classes = []

    if len(rcif.items)+len(rcif.loops) > 0:
        str_rcif_items = str(rcif.items)
        l_optional_classes = []
        for _item_class in L_ITEM_CLASS:
            item_obj = _item_class.from_cif(str_rcif_items)
            if item_obj is not None:
                l_optional_classes.append(_item_class)
                l_global_obj.append(item_obj)
                if _item_class not in l_defined_classes:
                    l_defined_classes.append(_item_class)
        for _loop_obj in rcif.loops:
            for _loop_class in L_LOOP_CLASS:
                loop_obj = _loop_class.from_cif(str(_loop_obj))
                if loop_obj is None:
                    pass
                elif len(loop_obj) != 0:
                    l_optional_classes.append(_loop_class)
                    l_global_obj.append(loop_obj)
                    if _loop_class not in l_defined_classes:
                        l_defined_classes.append(_loop_class)

    for _data in rcif.datas:
        flag = False
        
        str_data = str(_data)
        for _data_class in L_DATA_CLASS:
            data_obj = _data_class.from_cif(str_data)
            if data_obj is not None:
                l_global_obj.append(data_obj)
                if not(_data_class in l_defined_classes):
                    l_defined_classes.append(_data_class)
                flag = True
        if not(flag):
            str_data_items = str(_data.items)
            l_data_item_obj, l_data_loop_obj = [], []
            l_optional_classes = []
            for _item_class in L_ITEM_CLASS:
                item_obj = _item_class.from_cif(str_data_items)
                if item_obj is not None:
                    l_optional_classes.append(_item_class)
                    l_data_item_obj.append(item_obj)
            for _loop_obj in _data.loops:
                for _loop_class in L_LOOP_CLASS:
                    loop_obj = _loop_class.from_cif(str(_loop_obj))
                    if loop_obj is None:
                        pass
                    elif len(loop_obj) != 0:
                        l_optional_classes.append(_loop_class)
                        l_data_loop_obj.append(loop_obj)
            
            if ((len(l_data_item_obj) != 0) | (len(l_data_loop_obj) != 0)):
                _data_class = DataConstr
                data_obj = DataConstr(data_name=_data.name, optional_classes=list(set(l_optional_classes)))
                data_obj.optional_objs = l_data_item_obj + l_data_loop_obj
                l_global_obj.append(data_obj)
                l_defined_classes.append(_data_class)
            
    global_constr = GlobalConstr(global_name="", mandatory_classes=l_defined_classes)
    global_constr.mandatory_objs = l_global_obj 
    return global_constr


